@@ -9,9 +9,11 @@ Test suite for pyubx2
 import os
 import sys
 import unittest
+from importlib import import_module
 
 # inject local copy to avoid testing the installed version instead of the one in the repo
 sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
+currdir = os.path.dirname(__file__)
 import pyubx2
 print(f"Testing Local Version: {pyubx2.VERSION}")
 
@@ -26,11 +28,11 @@ if len(sys.argv) > 1:
 mainsuite = unittest.TestSuite()
 for modulename in [
         os.path.splitext(x)[0]
-        for x in os.listdir(os.path.dirname(__file__) or '.')
+        for x in os.listdir(currdir or '.')
         if x != __file__ and x.startswith("test_") and x.endswith(".py")
 ]:
     try:
-        module = __import__(modulename)
+        module = import_module(modulename)
     except ImportError as err:
         print(f"skipping {modulename}, {err}")
     else:
@@ -42,7 +44,7 @@ for modulename in [
 verbosity = 1
 if '-v' in sys.argv[1:]:
     verbosity = 2
-    print('-' * 78)
+print('-' * 70)
 
 # run the collected tests
 testRunner = unittest.TextTestRunner(verbosity=verbosity)
